#!/bin/sh
NAME="mountiso"

# -------------------------------------------------------------------------
function set_env {
# parameters:
# none
# -------------------------------------------------------------------------

# Mimic the look of installer when setting variables
  echo "--------------------------------------------"
  echo "Environment:"

  LOCAL="`kde-config --localprefix 2>/dev/null`"

  if ( test -d "$LOCAL" ) then
    LOCAL="$LOCAL"
  elif ( test -d "$HOME/.kde" ) then
    LOCAL="$HOME/.kde"
  else
    echo
    while ( test ! -d "$LOCAL" )
    do
      echo "Couldn't find KDE directory in $HOME."
      echo -n "Type the full path here or press \"Ctrl+C\" to abort: "
      read LOCAL
    done
  fi
  
  LOCAL="`dirname "$LOCAL"`/`basename "$LOCAL"`"
  echo -n "* KDE config folder   "
  echo $LOCAL

  DESKTOP="`kde-config --userpath desktop 2>/dev/null`"

  if ( test -d "$DESKTOP" ) then
    DESKTOP="$DESKTOP"
  elif ( test -d "$HOME/Desktop" ) then
    DESKTOP="$HOME/Desktop"
  else
    echo
    while ( test ! -d "$DESKTOP" )
    do
      echo "Couldn't find desktop directory in $HOME."
      echo -n "Type the full path here or press \"Ctrl+C\" to abort: "
      read DESKTOP
    done
  fi
  
  DESKTOP="`dirname "$DESKTOP"`/`basename "$DESKTOP"`"
  echo -n "* Desktop folder      "
  echo $DESKTOP
  echo "--------------------------------------------"
}

# -------------------------------------------------------------------------
function hi {
# parameters:
# - $1 = selected install mode (kdesu or sudo)
# -------------------------------------------------------------------------
  
  echo
  echo "      Mount-ISO (version using \"$1\")"
  echo
}

# -------------------------------------------------------------------------
function bye {
# parameters:
# none
# -------------------------------------------------------------------------
  
  echo
  echo "       Thank you for using Mount-ISO!"
  echo
}

# -------------------------------------------------------------------------
function install {
# parameters:
# - $1 = selected install mode (kdesu or sudo)
# -------------------------------------------------------------------------
  
  hi $1
  install_menu
  install_script $1
  bye
}

# -------------------------------------------------------------------------
function uninstall {
# parameters:
# none
# -------------------------------------------------------------------------
  
  SH="$LOCAL/share/apps/$NAME/"
  DESKTOP="$LOCAL/share/apps/konqueror/servicemenus/$NAME.desktop"
  
  echo
  echo -n "Uninstalling $NAME.sh:           "
  if ( test -d "$SH" )
  then
    rm -r "$SH" && echo "[  OK  ]" || { echo "[FAILED]"; exit 1; }
  else
    echo "[ NONE ]"
  fi
  echo -n "Uninstalling $NAME.desktop:      "
  if ( test -f "$DESKTOP" )
  then
    rm "$DESKTOP" && echo "[  OK  ]" || { echo "[FAILED]"; exit 1; }
  else
    echo "[ NONE ]"
  fi
  
}


# -------------------------------------------------------------------------
function install_menu {
# parameters:
# none
# -------------------------------------------------------------------------
  
  echo -n "Creating a folder for servicemenus: "
  if ( test ! -d "$LOCAL/share/apps/konqueror/servicemenus/" )
  then
    mkdir -p "$LOCAL/share/apps/konqueror/servicemenus/" 2>/dev/null
    test -d "$LOCAL/share/apps/konqueror/servicemenus" &&
    echo "[  OK  ]" || { echo "[FAILED]" ; exit 1; }
  else
    echo "[ SKIP ]"
  fi

  echo -n "Installing $NAME.desktop:        "

  echo '[Desktop Entry]
Actions=MountISOImage;UmountISOImage;
Encoding=UTF-8
ServiceTypes=application/x-iso,inode/directory,inode/ISO-image
X-KDE-Submenu=Manage ISO
X-KDE-Submenu[br]=Gest� da ISO
X-KDE-Submenu[cs]=ÃÂÃÂ­dit ISO
X-KDE-Submenu[da]=ISO hУЅndtering
X-KDE-Submenu[de]=ISO-Filemanagement
X-KDE-Submenu[es]=Gestion de ISO
X-KDE-Submenu[fr]=Gestion des ISO
X-KDE-Submenu[hu]=ISO képmásfájl
X-KDE-Submenu[it]=Gestione ISO
X-KDE-Submenu[pl]=Zarzdzanie obrazem ISO
X-KDE-Submenu[pt_BR]=Gest� da ISO
X-KDE-Submenu[ru]=ISO образ
X-KDE-Submenu[sk]=RiadiÃÂ¥ ISO

[Desktop Action MountISOImage]
Icon=cdimage
Name=Mount ISO Image
Name[br]=Montar a imagem ISO
Name[cs]=PÃÂipojit ISO obraz
Name[da]=MontУЉr CD-billede
Name[de]=ISO-File einbinden
Name[es]=Montar imagen ISO
Name[fr]=Monter l`image ISO
Name[hu]=ISO képmásfájl csatolása
Name[it]=Monta immagine ISO
Name[pl]=Zamontuj obraz ISO
Name[pt_BR]=Montar a imagem ISO
Name[ru]=Смонтировать ISO образ
Name[sk]=PripojiÃÂ¥ ISO obraz
Exec=`kde-config --localprefix`share/apps/'$NAME'/'$NAME'.sh "%U" mount

[Desktop Action UmountISOImage]
Icon=cdimage
Name=Unmount ISO Image
Name[br]=Desmontar a imagem ISO
Name[cs]=Odpojit ISO obraz
Name[da]=AfmontУЉr CD-billede
Name[de]=ISO-File Einbindung l�en
Name[es]=Desmontar imagen ISO
Name[fr]=DÑmonter l`image ISO

Name[hu]=ISO képmásfájl leválasztás
Name[it]=Smonta immagine ISO
Name[pl]=Odmontuj obraz ISO
Name[pt_BR]=Desmontar a imagem ISO
Name[ru]=Отмонтировать ISO образ
Name[sk]=OdpojiÃÂ¥ ISO obraz
Exec=`kde-config --localprefix`share/apps/'$NAME'/'$NAME'.sh "%U" unmount
' > ${LOCAL}/share/apps/konqueror/servicemenus/$NAME.desktop && echo "[  OK  ]" || { echo "[FAILED]"; exit 1; }
}

# -------------------------------------------------------------------------
function install_script {
# parameters:
# - $1 = install directory (usually "~/.kde")
# -------------------------------------------------------------------------
  
  case $1 in
    sudo)
      MOUNT='sudo mount -t iso9660 -o loop,ro,nodev,noexec,nosuid "$1" "$mountdir"'
      UNMOUNT='sudo umount "$mountdir"'
      ;;
    kdesu)
      MOUNT='kdesu -c "mount -t iso9660 -o loop,ro,nodev,noexec,nosuid \"$1\"  \"$mountdir\""'
      UNMOUNT='kdesu -c "umount \"$mountdir\""'
      ;;
  esac
  
  echo -n "Installing $NAME.sh:             "
  if ( test -d "$LOCAL/share/apps/$NAME/" )
  then
    rm -r "$LOCAL/share/apps/$NAME"
  fi
  
  mkdir -p "$LOCAL/share/apps/$NAME" 2>/dev/null
  test -d "$LOCAL/share/apps/$NAME" &&
  echo '#!/bin/sh
bsname="`basename "$1"`"

if ( `echo "$bsname" | grep "Mount-ISO" > /dev/null` ) then
  mountdir="$1"
else
  mountdir="'$DESKTOP'/Mount-ISO ($bsname)"
fi

function err {
  case $1 in
   1)
     ERROR_MESSAGE="\"$2\" is a directory, not an ISO-image!"
     ;;
   2)
     ERROR_MESSAGE="\"$2\" is already mounted!"
     ;;
   3)
     ERROR_MESSAGE="\"$2\" could not be created!"
     ;;
   4)
     ERROR_MESSAGE="\"$2\" is not mounted!"
     ;;
   5)
     ERROR_MESSAGE="Folder \"$2\" exists but ISO is not mounted!"
     ;;
   6)
     ERROR_MESSAGE="Could not mount \"$2\"! Check your system settings."
     ;;
   *)
     return 0
     ;;
  esac
  dcop kio_uiserver UIServer messageBox 1 5 "ERROR: $ERROR_MESSAGE" "Mount ISO image" "Ok" ""
  exit 0
}

function check_mount {
  if ( `mount | grep "$1" > /dev/null` ) then
    return 0
  else
    return 1
  fi
}

case "$2" in
  mount)
    if ( test -d "$1" ) then
      err 1 "$mountdir"
    elif (test -d "$mountdir") then
      check_mount "$mountdir" &&
      err 2 "$mountdir"
    else
      mkdir -p "$mountdir"
    fi
    test -d "$mountdir" &&
    '$MOUNT'
    if ( check_mount "$mountdir" ) then
      kfmclient openURL "$mountdir"
    else
      rmdir "$mountdir"
      err 6 "$mountdir"
    fi
    ;;
  unmount)
    if (test ! -d "$mountdir") then
      err 4 "$mountdir"
    elif (check_mount "$mountdir") then
      '$UNMOUNT' &&
      rmdir "$1" ||
      rmdir "$mountdir"
    else
      err 5 "$mountdir"
    fi
    ;;
esac
' > ${LOCAL}/share/apps/$NAME/$NAME.sh &&
  chmod +x ${LOCAL}/share/apps/$NAME/$NAME.sh && echo "[  OK  ]" || { echo "[FAILED]"; exit 1; }
}

# -------------------------------------------------------------------------
function setup_sudo {
# parameters:
# none
# -------------------------------------------------------------------------
  
  echo
  echo "Only root can edit sudo config - type root password here"
# add 1 line to /etc/sudoers to give users permission to (u)mount loopback devices in their home directory
  su -c "echo \"%users ALL=NOPASSWD:/bin/mount -t iso9660 -o ro\,loop\,nodev\,noexec\,nosuid */Mount-ISO*,/bin/umount */Mount-ISO*\" >> /etc/sudoers && echo && echo \"Success! Press Enter to continue\" && read"
}

# -------------------------------------------------------------------------
function reset_sudo {
# parameters:
# none
# -------------------------------------------------------------------------
  
  echo
  echo "Only root can edit sudo config - type root password here"
# copy the content of /etc/sudoers to a buffer first and then
# delete all lines in /etc/sudoers to give users permission to
# (u)mount loopback devices in their home directory
  su -c 'SUDOERS=$(</etc/sudoers) && echo "$SUDOERS" | sed -e "/[mM]ount-[iI][sS][oO]/d" > /etc/sudoers && echo && echo "Success! Press Enter to continue" && read'
}

# -------------------------------------------------------------------------
#
# MAIN
#
# -------------------------------------------------------------------------

# clear the screen
clear
# initialize and detect environment variables
set_env

# show main menu
while :
do
# clear the screen
  clear  
  echo "--------------------------------------------"
  echo "Environment:"
  echo -n "* KDE config folder   "
  echo $LOCAL
  echo -n "* Desktop folder      "
  echo $DESKTOP
  echo "--------------------------------------------"
  echo "Select action:"
  echo " 1 - Install Mount-ISO using sudo"
  echo " 2 - Install Mount-ISO using kdesu"
  echo " 3 - Uninstall Mount-ISO"
  echo " 4 - Setup sudo config [*]"
  echo " 5 - Reset sudo config [*]"
  echo " 0 - Quit the installer"
  echo "--------------------------------------------"
  echo "[*] - requires root privileges"
  echo "--------------------------------------------"
  echo -n "Select options 1-5 or 0 to quit: "
  read line
# act accordingly
  case "$line" in
    1)
      install sudo
      exit 0
      ;;
    2) 
      install kdesu
      exit 0
      ;;
    3)
      uninstall
      bye
      exit 0
      ;;
    4)
      setup_sudo
      continue
      ;;
    5)
      reset_sudo
      clear
      continue
      ;;
    0)
      bye
      exit 0
      ;;
    *)
      echo
      ;;
  esac
  
done
